library Sage;

{ Important note about DLL memory management: ShareMem must be the
  first unit in your library's USES clause AND your project's (select
  Project-View Source) USES clause if your DLL exports any procedures or
  functions that pass strings as parameters or function results. This
  applies to all strings passed to and from your DLL--even those that
  are nested in records and classes. ShareMem is the interface unit to
  the BORLNDMM.DLL shared memory manager, which must be deployed along
  with your DLL. To avoid using BORLNDMM.DLL, pass string information
  using PChar or ShortString parameters. }

uses
  SysUtils,
  Classes,
  IdBaseComponent,
  IdComponent,
  IdTCPConnection,
  IdTCPClient,
  DateUtils,
  Windows,
  SageThread in 'SageThread.pas',
  Utils in 'Utils.pas';

{$R *.res}

var
  Value: string;

// Smartie will call this when the plugin is 1st loaded
// This function is optional
Procedure SmartieInit; stdcall;
begin
  FSageThread := TSageThread.Create;
end;

// Smartie will call this just before the plugin is unloaded
// This function is optional
Procedure SmartieFini; stdcall;
begin
  FSageThread.Free;
end;

// Define the minimum interval that a screen should get fresh data from our
// plugin.
// The actual value used by Smartie will be the higher of this value and
// of the 'dll check interval' setting
// on the Misc tab.  [This function is optional, Smartie will assume
// 300ms if it is not provided.]
Function GetMinRefreshInterval: Integer; stdcall;
begin
  result := 300; // 300 ms
end;

function function1(param1:pchar;param2:pchar):pchar; stdcall;
// Type/Rec devname
// Param1 0 = Playing, 1... = Recording number
// Param2 0 = As text, 1 = 0 for not playing, 1 for playing
begin
  if Assigned(FSageThread) then
    Value := FSageThread.GetTypeDevname(sStrToInt(param1), param2);
  Result := PChar(Value);
end;

function function2(param1:pchar;param2:pchar):pchar; stdcall;
// Show
// Param1 0 = Playing, 1... = Recording number
begin
  if Assigned(FSageThread) then
    Value := FSageThread.GetShow(sStrToInt(param1));
  Result := PChar(Value);
end;

Function function3(param1:pchar;param2:pchar):pchar; stdcall;
// Episode
// Param1 0 = Playing, 1... = Recording number
begin
  if Assigned(FSageThread) then
    Value := FSageThread.GetEpisode(sStrToInt(param1));
  Result := PChar(Value);
end;

Function function4(param1:pchar;param2:pchar):pchar; stdcall;
// Artist
begin
  if Assigned(FSageThread) then
    Value := FSageThread.GetArtist;
  Result := PChar(Value);
end;

Function function5(param1:pchar;param2:pchar):pchar; stdcall;
// Channel
// Param1 0 = Playing, 1... = Recording number
// Param2 0/'' = Both, 1 = Channel number, 2 = Channel name
begin
  if Assigned(FSageThread) then
    Value := FSageThread.GetChannel(sStrToInt(param1), Trim(Param2));
  Result := PChar(Value);
end;

Function function6(param1:pchar;param2:pchar):pchar; stdcall;
// StartTime
// Param1 0 = Playing, 1... = Recording number
// Param2 = Formatting ex yyyy-mm-dd hh:nn:ss
begin
  if Assigned(FSageThread) then
    Value := FSageThread.GetStartTime(sStrToInt(param1), Trim(Param2));
  Result := PChar(Value);
end;

Function function7(param1:pchar;param2:pchar):pchar; stdcall;
// EndTime
// Param1 0 = Playing, 1... = Recording number
// Param2 = Formatting ex yyyy-mm-dd hh:nn:ss,
// if param2 contains a # this is used to separate param2 in two parameters
// The second part (after #) is the format
// The first part is l for having length of show instead of end time and
// ls is show length in seconds
begin
  if Assigned(FSageThread) then
    Value := FSageThread.GetEndTime(sStrToInt(param1), Trim(Param2));
  Result := PChar(Value);
end;

Function function8(param1:pchar;param2:pchar):pchar; stdcall;
// Curpos
// Param2 = Formatting ex h:nn:ss
// if param2 contains a # this is used to separate param2 in two parameters
// The second part (after #) is the format
// If the first part is s the position is given in seconds from the start
begin
  if Assigned(FSageThread) then
    Value := FSageThread.GetCurPos(Trim(Param2));
  Result := PChar(Value);
end;

Function function9(param1:pchar;param2:pchar):pchar; stdcall;
// State
begin
  if Assigned(FSageThread) then
    Value := FSageThread.GetState;
  Result := PChar(Value);
end;

Function function10(param1:pchar;param2:pchar):pchar; stdcall;
// Num rec devs
begin
  if Assigned(FSageThread) then
    Value := FSageThread.GetRecDevs;
  Result := PChar(Value);
end;

// don't forget to export the funtions, else nothing works :)
exports
  function1,
  function2,
  function3,
  function4,
  function5,
  function6,
  function7,
  function8,
  function9,
  function10,
  SmartieInit,
  SmartieFini,
  GetMinRefreshInterval;

begin
end.


