package net.sf.sageplugins.getstatus;


import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;


import sage.SageTV;

/**
 * @author Niel Markwick
 *
 * Simple TCP server to supply Sage status information
 */
public class GetStatus implements Runnable {
	private static final String VERSION="1.2";
	private static final int DEF_PORT=20000;
	private static final int DEF_TIMEOUT=60000;
	private boolean hasApi=true;
	LinkedList clientlist;
	/**
	 * Wrapper for Sage Api catching all exceptions
	 * @param api
	 * @param args
	 * @return
	 */
	private Object SageApi(String api, Object[] args){
		if ( hasApi ){
			try {
				return SageTV.api(api,args);
			} catch (Throwable e) {
				System.out.println("SageApi("+api+") threw: "+e.toString());
				if ( e.getCause()!=null)
					System.out.println("caused by "+e.getCause().toString());
			}
		}
		return null;
	}

	public GetStatus() {
	}

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	class Client {
		Client(SocketChannel socket) { 
			this.socket=socket; 
			lastupdatetime=System.currentTimeMillis();
		}
		long lastupdatetime;
		SocketChannel socket;
	};
	public void run() {
		System.out.println("GetStatus: Started GetStatus version "+VERSION+" monitor thread");
		try { 
			Object retval=SageApi("GetProperty",new Object[]{"nielm/getstatus/port",new Integer(DEF_PORT)});
			int server_port;
			if ( retval!=null)
				server_port=(Integer.parseInt((String)retval));
			else
				server_port=DEF_PORT;
			retval=SageApi("GetProperty",new Object[]{"nielm/getstatus/bind_addr","127.0.0.1"});
			String bind_addrstr;
			if ( retval!=null)
				bind_addrstr=(String)retval;
			else
				bind_addrstr="127.0.0.1";
			
			InetAddress bind_addr;
			try {
				bind_addr = InetAddress.getByName(bind_addrstr);
			}catch (UnknownHostException e){
				System.out.println("GetStatus: cannot get local address "+e.toString());
				throw e;
			}
			
			long timeoutms;
			retval=SageApi("GetProperty",new Object[]{"nielm/getstatus/timeoutms","60000"});
			if ( retval!=null)
				timeoutms=(Long.parseLong((String)retval));
			else
				timeoutms=DEF_TIMEOUT;
			
			Selector sel=Selector.open();
			ServerSocketChannel server=ServerSocketChannel.open();
			clientlist=new LinkedList();
			try {
				InetSocketAddress addr=new InetSocketAddress(bind_addr,server_port);
				server.socket().bind(addr);
				server.configureBlocking(false);
			}catch (IOException e){
				System.out.println("GetStatus: Cannot listen on port "+Integer.toString(server_port)+"\r\n"
						+e.toString());
				throw e;
			}
			server.register(sel,SelectionKey.OP_ACCEPT);
			System.out.println("GetStatus: Listening on "+bind_addrstr+":"+Integer.toString(server_port));
			while ( true ) {
				sel.select(1000); // 1 second timeout
								
				Set readyKeys = sel.selectedKeys();
				Iterator it=readyKeys.iterator();
				while ( it.hasNext()) {
					SelectionKey key=(SelectionKey)it.next();
					it.remove();
					if ( key.isAcceptable()){
						SocketChannel socket=null;
						try {
							ServerSocketChannel ssc = (ServerSocketChannel) key.channel();
							socket = (SocketChannel) ssc.accept();
							socket.configureBlocking(false);
							socket.register(sel,SelectionKey.OP_READ);
							System.out.println("GetStatus: accepted connection");
							clientlist.add(new Client(socket));
							SendData(socket);
						} catch ( Throwable e) {
							System.out.println("GetStatus: Exception when accepting client"+Integer.toString(server_port)+"\r\n"
									+e.toString());
							e.printStackTrace(System.out);
							if ( socket != null)
								socket.close();
						}
					}
					if ( key.isReadable()){
						SocketChannel socket = (SocketChannel)key.channel();
						ByteBuffer buf = ByteBuffer.allocate(1024);
						try
						{
				            int nBytes = socket.read(buf);
				            if ( nBytes<0) {
				            	System.out.println("GetStatus: closed connection");
				            	// find client in list
				            	Client client=FindClient(socket);
				            	if ( client != null)
				            		clientlist.remove(client);
				            	socket.close();
				            } else {
				            	SendData(socket);
				            }
				            
						} catch (Throwable e){
							System.out.println("GetStatus: Exception when handling client"+Integer.toString(server_port)+"\r\n"
									+e.toString());
							e.printStackTrace(System.out);
							// find client in list
			            	Client client=FindClient(socket);
			            	if ( client != null)
			            		clientlist.remove(client);
		            		socket.close();
						}
					}
				}
				it=clientlist.iterator();
				while ( it.hasNext() ) {
					Client client=(Client)it.next();
					if ( client.lastupdatetime+timeoutms < System.currentTimeMillis()){
						System.out.println("GetStatus: timed out connection");
						// iterate before removing
						clientlist.remove(client);
						client.socket.close();
						// restart iterator
						it=clientlist.iterator();
					}
				}
			}	
		} catch (Throwable e){
			System.out.println("GetStatus: Status monitor thread threw: "+e.toString());
			e.printStackTrace(System.out);
		}
		System.out.println("GetStatus: Ended GetStatus monitor thread");
	}

	/**
	 * Test stub
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.println(
				"Sage GetStatus class V"+VERSION+"\r\n" +
				"Running in test mode: no status available\r\n" +
				"Needs to run in Sage to retrieve actual status");
		GetStatus status=new GetStatus();
		status.hasApi=false;
		Thread thread = new Thread( status, "GetStatus handler" );
        thread.start();
	}
	
	Client FindClient(SocketChannel socket){
		Iterator clientit=clientlist.iterator();
    	while ( clientit.hasNext()) {
			Client client=(Client)clientit.next();
			if ( client.socket == socket)
				return client;
    	}
    	return null;
	}

	// cache to hold data for current media file to prevent too many lookups in Sage's DB
	Object currMediaFile=null;
	String currMFtitle="";
	String currMFepisode="";
	String currMFtype="";
	String currMFartist="";
	String currMFchannel="";
	long currMFstarttime=0;
	long currMFendtime=0;
	
	// cache for each capture device
	class CapDevStatus {
		Object captureDevice=null;
		String captureDeviceName="";
		Object mediaFile=null;
		String title="";
		String episode="";
		String channel="";
		long starttime=0;
		long endtime=0;
	}
	// check number of capture devices every 60 seconds
	int numCapDevs=-1;
	long lastCapDevCheckMillis=0;
	CapDevStatus[] capDevStatusArr=null;
	
	
	
	void SendData(SocketChannel socket) 
	throws Throwable 
	{
		Client client=FindClient(socket);
    	if ( client != null)
    		client.lastupdatetime=System.currentTimeMillis();
    	

		System.out.println("GetStatus: sending data");
		StringWriter outstr=new StringWriter();
		PrintWriter out=new PrintWriter(outstr);
		
		if ( hasApi) {
			// Get+print status data
			//
			// now_playing_type: [tv|music|dvd|nothing]
			// now_playing_show: [showname|albumname]
			// now_playing_episode: [episodename|trackname]
			// now_playing_artist: [artistname]
			// now_playing_channel: [channel number - name]
			// now_playing_starttime: [milliseconds since 1970 epoch]
			// now_playing_endtime: [milliseconds since 1970 epoch]
			// now_playing_currpos: [milliseconds since 1970 epoch]
			// now_playing_state: [play|paused|stopped]
			// num_recording_devs [n]
			// recording_1_dev: [devicename]
			// recording_1_chan: [channel number - name]
			// recording_1_show: [showname]
			// recording_1_episode: [showname]
			// recording_1_starttime: [milliseconds since 1970 epoch]
			// recording_1_endtime: [milliseconds since 1970 epoch]

			
			// get current media file!
			Boolean hasMf=(Boolean)SageApi("HasMediaFile",null);
			Object Mf=null;
			if ( hasMf.booleanValue() ) {
				Mf=SageApi("GetCurrentMediaFile",null);
			} else {
				Mf=null;
				currMediaFile=null;
			}
			if ( Mf!=null){
				if ( currMediaFile != Mf) {
					// Update cached data
					System.out.println("Updating now playing cache");
					currMediaFile=Mf;
					currMFtitle="";
					currMFepisode="";
					currMFartist="";
					currMFchannel="";
					currMFstarttime=0;
					currMFendtime=0;
					
					if ( ((Boolean)SageApi("IsMusicFile",new Object[]{Mf})).booleanValue()) {
						currMFtype="music";
						currMFartist=(String)SageApi("GetPeopleInShowInRole",new Object[]{Mf, "Artist"});
						currMFstarttime=((Long)SageApi("GetAiringStartTime",new Object[]{Mf})).longValue();
						currMFendtime=((Long)SageApi("GetAiringEndTime",new Object[]{Mf})).longValue();
					}
					else if ( ((Boolean)SageApi("IsTVFile",new Object[]{Mf})).booleanValue()){
						currMFtype="tv";
						String chnum=(String)SageApi("GetChannelNumber",new Object[]{Mf});
						String chnm=(String)SageApi("GetChannelName",new Object[]{Mf});
						if ( chnum!=null)
							currMFchannel=chnum+" - ";
						if ( chnm!=null)
							currMFchannel+=chnm;
						currMFstarttime=((Long)SageApi("GetAiringStartTime",new Object[]{Mf})).longValue();
						currMFendtime=((Long)SageApi("GetAiringEndTime",new Object[]{Mf})).longValue();
					}
					else if ( ((Boolean)SageApi("IsDVD",new Object[]{Mf})).booleanValue()){
						currMFtype="dvd";
						currMFendtime=((Long)SageApi("GetMediaDuration",null)).longValue();
					}
					else {
						currMFtype="video";
						currMFstarttime=((Long)SageApi("GetAiringStartTime",new Object[]{Mf})).longValue();
						currMFendtime=((Long)SageApi("GetAiringEndTime",new Object[]{Mf})).longValue();
					}
					// get common information
					currMFtitle=(String)SageApi("GetAiringTitle",new Object[]{Mf});
					currMFepisode=(String)SageApi("GetShowEpisode",new Object[]{Mf});
				}
				// get current position
				long endtime=0;
				long starttime=0;
				long currpos=0;
				if ( currMFtype.equals("music") || currMFtype.equals("video") ) {
					endtime=currMFendtime-currMFstarttime;
					currpos=((Long)SageApi("GetMediaTime",new Object[]{Mf})).longValue();
					currpos-=currMFstarttime;
				} 
				else if ( currMFtype.equals("dvd") ){ 
					currpos=((Long)SageApi("GetMediaTime",new Object[]{Mf})).longValue();
					endtime=currMFendtime;
				}
				else if ( currMFtype.equals("tv") ){ 
					currpos=((Long)SageApi("GetMediaTime",new Object[]{Mf})).longValue();
					endtime=currMFendtime;
					starttime=currMFstarttime;
				}
				
				out.println("now_playing_type: "+currMFtype);
				out.println("now_playing_show: "+currMFtitle);
				out.println("now_playing_episode: "+currMFepisode);
				out.println("now_playing_artist: "+currMFartist);
				out.println("now_playing_channel: "+currMFchannel);
				out.println("now_playing_starttime: "+Long.toString(starttime));
				out.println("now_playing_endtime: "+Long.toString(endtime));
				out.println("now_playing_currpos: "+Long.toString(currpos));
				if ( ((Boolean)SageApi("IsPlaying",null)).booleanValue())
					out.println("now_playing_state: play");
				else
					out.println("now_playing_state: paused");
			}else{
				// MF=null
				out.println(
				 "now_playing_type: nothing\r\n"+
				 "now_playing_show: \r\n"+
				 "now_playing_episode: \r\n"+
				 "now_playing_artist: \r\n"+
				 "now_playing_channel: \r\n"+
				 "now_playing_starttime: \r\n"+
				 "now_playing_endtime: \r\n"+
				 "now_playing_currpos: \r\n"+
				 "now_playing_state: stopped");
			}
			
			if ( ( lastCapDevCheckMillis + 60000) < System.currentTimeMillis()) {
				// do the capture device statuses
				Object captureDevInputs=SageApi("GetConfiguredCaptureDeviceInputs",null);
				Map capDevInputMap=(Map)SageApi("GroupByMethod",new Object[]{captureDevInputs,"GetCaptureDeviceForInput"});
				Set captureDevs=null;
				if ( capDevInputMap!= null){
					captureDevs=capDevInputMap.keySet();
					if ( captureDevs.size() != numCapDevs)
					{
						// different number of capture devices 
						numCapDevs=captureDevs.size();
						capDevStatusArr=new CapDevStatus[numCapDevs];
					}
					int capdevnum=0;
					for (Iterator i=captureDevs.iterator();i.hasNext();) {
						if ( capDevStatusArr[capdevnum]==null)
							capDevStatusArr[capdevnum]=new CapDevStatus();
						capDevStatusArr[capdevnum].captureDevice=i.next();
						capDevStatusArr[capdevnum].captureDeviceName=capDevStatusArr[capdevnum].captureDevice.toString();
						capdevnum++;
					}
				} else {
					System.out.println("CapDevInputMap is NULL");
					numCapDevs=0;
				}
				lastCapDevCheckMillis=System.currentTimeMillis();
			}
			
			out.println("num_recording_devs: "+numCapDevs);
			
			for (int devnum=0;devnum<numCapDevs;devnum++){
				Object capDev=capDevStatusArr[devnum].captureDevice;
				out.println("recording_"+Integer.toString(devnum+1)+"_dev: "+capDevStatusArr[devnum].captureDeviceName);
				
				
				Mf=SageApi("GetCaptureDeviceCurrentRecordFile",new Object[]{capDev});
				if ( Mf != capDevStatusArr[devnum].mediaFile) {

					// update cache
					System.out.println("Updating recording device "+Integer.toString(devnum+1)+" cache");
					capDevStatusArr[devnum].mediaFile=Mf;
					
					if ( Mf == null ) {
						capDevStatusArr[devnum].title="";
						capDevStatusArr[devnum].episode="";
						capDevStatusArr[devnum].channel="";
						capDevStatusArr[devnum].starttime=0;
						capDevStatusArr[devnum].endtime=0;
					} else {
						capDevStatusArr[devnum].channel="";
						String chnum=(String)SageApi("GetChannelNumber",new Object[]{Mf});
						String chnm=(String)SageApi("GetChannelName",new Object[]{Mf});
						if ( chnum!=null)
							capDevStatusArr[devnum].channel=chnum+" - ";
						if ( chnm!=null)
							capDevStatusArr[devnum].channel+=chnm;
						
						capDevStatusArr[devnum].title=(String)SageApi("GetAiringTitle",new Object[]{Mf});
						capDevStatusArr[devnum].episode=(String)SageApi("GetShowEpisode",new Object[]{Mf});
						capDevStatusArr[devnum].starttime=((Long)SageApi("GetScheduleStartTime",new Object[]{Mf})).longValue();
						capDevStatusArr[devnum].endtime=((Long)SageApi("GetScheduleEndTime",new Object[]{Mf})).longValue();
					}
				}
				
				// Ouput data
				if ( capDevStatusArr[devnum].mediaFile != null ) {
					out.println("recording_"+Integer.toString(devnum+1)+"_show: "+
							capDevStatusArr[devnum].title);
					out.println("recording_"+Integer.toString(devnum+1)+"_episode: "+
							capDevStatusArr[devnum].episode);
					out.println("recording_"+Integer.toString(devnum+1)+"_chan: "+
							capDevStatusArr[devnum].channel);
					out.println("recording_"+Integer.toString(devnum+1)+"_starttime: "+
							Long.toString(capDevStatusArr[devnum].starttime));
					out.println("recording_"+Integer.toString(devnum+1)+"_endtime: "+
							Long.toString(capDevStatusArr[devnum].endtime));
				} else {
					out.println(
						"recording_"+Integer.toString(devnum+1)+"_show: \r\n"+
						"recording_"+Integer.toString(devnum+1)+"_episode: \r\n"+
						"recording_"+Integer.toString(devnum+1)+"_chan: \r\n"+
						"recording_"+Integer.toString(devnum+1)+"_starttime: \r\n"+
						"recording_"+Integer.toString(devnum+1)+"_endtime:");
				}
			}
		} else {
			out.println("Test mode -- no status available");
		}
		out.println();
		out.flush();
		out.close();
		ByteBuffer buffer = ByteBuffer.wrap((outstr.toString()).getBytes());
		socket.write(buffer);
	}
}
