unit SageThread;

interface

uses
  Classes, Windows, SyncObjs, SysUtils, DateUtils,
  IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, Inifiles,
  Utils;

type
  TSageThread = class(TThread)
  private
    FSageAddress: string;
    FSagePort: Integer;
    FDefaultPollInterval: Integer;
    FPollInterval: Integer;

    FCriticalSection: TCriticalSection;

    FLastStatusInfo: string; // Needed to alternate status
                              // This is needed becouse actions
                              // in LCD Smartie only fires on changes
                              // and we want to keep it locked to a screen
  public
    Data: TStringList;

    function GetTypeDevname(RecNo: Integer; ResultType: string): string;
    function GetShow(RecNo: Integer): string;
    function GetEpisode(RecNo: Integer): string;
    function GetArtist: string;
    function GetChannel(RecNo: Integer; Part: string): string;
    function GetStartTime(RecNo: Integer; Format: string): string;
    function GetEndTime(RecNo: Integer; Format: string): string;
    function GetCurPos(Format: string): string;
    function GetState: string;
    function GetRecDevs: string;

    procedure Execute; override;

    constructor Create;
    destructor Destroy; override;
  end;

var
  FSageThread: TSageThread = nil;


implementation

{ TSageThread }

constructor TSageThread.Create;
var
  IniFile: TIniFile;
begin
  inherited Create(True);
  Data := TStringList.Create;
  FCriticalSection := TCriticalSection.Create;
  try
    IniFile := TIniFile.Create('Sage.ini');
    FSageAddress := IniFile.ReadString('Sage', 'IPAddress', '127.0.0.1');
    FSagePort := IniFile.ReadInteger('Sage', 'Port', 20000);
    FDefaultPollInterval  := iniFile.ReadInteger('Sage', 'PollInterval', 500);
    FPollInterval := FDefaultPollInterval;
    IniFile.Free;
  except
    FSageAddress := '127.0.0.1';
    FSagePort := 20000;
    FPollInterval := 500;
  end;
  Resume;
end;

destructor TSageThread.Destroy;
begin
  Data.Free;
  FCriticalSection.Free;
  inherited;
end;

procedure TSageThread.Execute;
var
  SageTCP: TIdTCPClient;
begin
  inherited;

  SageTCP := TIdTCPClient.Create(nil);
  SageTCP.Host := FSageAddress;
  SageTCP.Port := FSagePort;

  while not Terminated do
  begin
    if not SageTCP.Connected then
    begin
      try
        SageTCP.Connect;
        FPollInterval := FDefaultPollInterval;
      except
        FPollInterval := 5000;
      end;
    end;

    if SageTCP.Connected then
    begin
      try
        SageTCP.Write(' ');
        Sleep(5);
        try
          FCriticalSection.Enter;
          Data.Text := SageTCP.CurrentReadBuffer;
        finally
          FCriticalSection.Leave;
        end;
      except
        Data.Clear;
        SageTCP.Disconnect;
      end;
    end;
    sleep(FPollInterval);
  end;

  try
    if SageTCP.Connected then
      SageTCP.Disconnect;
  except
  end;

  SageTCP.Free;
end;

function TSageThread.GetArtist: string;
var
  RowData: string;
begin
  try
    FCriticalSection.Enter;
    if 3 < Data.Count then
      RowData := Data[3]
    else
      RowData := '';
  finally
    FCriticalSection.Leave;
  end;
  Result := Trim(Copy(RowData, Pos(':', RowData)+1, MAXINT));
end;

function TSageThread.GetEpisode(RecNo: Integer): string;
var
  RowData: string;
  Row: Integer;
begin
  Row := 2;
  if RecNo > 0 then
    Row := 12 + (RecNo-1) * 6;
  try
    FCriticalSection.Enter;
    if Row < Data.Count then
      RowData := Data[Row]
    else
      RowData := '';
  finally
    FCriticalSection.Leave;
  end;
  Result := Trim(Copy(RowData, Pos(':', RowData)+1, MAXINT));
end;

function TSageThread.GetChannel(RecNo: Integer; Part: string): string;
var
  RowData: string;
  Row: Integer;
begin
  Row := 4;
  if RecNo > 0 then
    Row := 13 + (RecNo-1) * 6;
  try
    FCriticalSection.Enter;
    if Row < Data.Count then
      RowData := Data[Row]
    else
      RowData := '';
  finally
    FCriticalSection.Leave;
  end;
  RowData := Trim(Copy(RowData, Pos(':', RowData)+1, MAXINT));
  if (Part = '1') then
    Result := Trim(Copy(RowData, 1, Pos('-', RowData)-1))
  else if (Part = '2') then
    Result := Trim(Copy(RowData, Pos('-', RowData)+1, MAXINT))
  else
   Result := RowData;
end;

function TSageThread.GetShow(RecNo: Integer): string;
var
  RowData: string;
  Row: Integer;
begin
  Row := 1;
  if RecNo > 0 then
    Row := 11 + (RecNo-1) * 6;
  try
    FCriticalSection.Enter;
    if Row < Data.Count then
      RowData := Data[Row]
    else
      RowData := '';
  finally
    FCriticalSection.Leave;
  end;
  Result := Trim(Copy(RowData, Pos(':', RowData)+1, MAXINT));
end;

function TSageThread.GetTypeDevname(RecNo: Integer; ResultType: string): string;
var
  RowData: string;
  Row: Integer;
begin
  Row := 0;
  if RecNo > 0 then
    Row := 10 + (RecNo-1) * 6;
  try
    FCriticalSection.Enter;
    if Row < Data.Count then
      RowData := Data[Row]
    else
      RowData := '';
  finally
    FCriticalSection.Leave;
  end;
  RowData := Trim(Copy(RowData, Pos(':', RowData)+1, MAXINT));
  if Trim(ResultType) <> '1' then
    Result := RowData
  else
  begin
    if (RowData = '') or (AnsiLowerCase(Trim(RowData)) = 'nothing') then
    begin
      Result := '0';
    end
    else
    begin
      if FLastStatusInfo = '1' then
      begin
        Result := '0';
        FLastStatusInfo := '0';
      end
      else
      begin
        Result := '1';
        FLastStatusInfo := '1';
      end;
    end;
  end;
end;

function TSageThread.GetState: string;
var
  RowData: string;
begin
  try
    FCriticalSection.Enter;
    if 8 < Data.Count then
      RowData := Data[8]
    else
      RowData := '';
  finally
    FCriticalSection.Leave;
  end;
  Result := Trim(Copy(RowData, Pos(':', RowData)+1, MAXINT));
end;

function TSageThread.GetRecDevs: string;
var
  RowData: string;
begin
  try
    FCriticalSection.Enter;
    if 9 < Data.Count then
      RowData := Data[9]
    else
      RowData := '';
  finally
    FCriticalSection.Leave;
  end;
  Result := Trim(Copy(RowData, Pos(':', RowData)+1, MAXINT));      
end;

function TSageThread.GetCurPos(Format: string): string;
var
  RowData1: string;
  RowData2: string;
  UT: Int64;
  T1, T2: TDateTime;
  PosInSeconds: Boolean;
begin
  PosInSeconds := False;
  if Format = '' then
    Format := 'h:nn:ss';
  if Pos('#', Format) > 0 then
  begin
    if AnsiLowerCase(Format[1]) = 's' then
      PosInSeconds := True;
    Format := Copy(Format, Pos('#', Format)+1, MAXINT);
  end;
  try
    FCriticalSection.Enter;
    if 7 < Data.Count then
    begin
      RowData1 := Data[7];
      RowData2 := Data[5];
    end
    else
    begin
      RowData1 := '';
      RowData2 := '';
    end;
  finally
    FCriticalSection.Leave;
  end;
  RowData1 := Trim(Copy(RowData1, Pos(':', RowData1)+1, MAXINT));
  RowData1 := Copy(RowData1, 1, Length(RowData1)-3); // Remove milliseconds
  RowData2 := Trim(Copy(RowData2, Pos(':', RowData2)+1, MAXINT));
  if Length(RowData2) > 3 then
    RowData2 := Copy(RowData2, 1, Length(RowData2)-3); // Remove milliseconds
  try
    if RowData1 <> '' then
    begin
      UT := StrToInt64(RowData1);
      T1 := UnixToDateTime(UT);
      UT := StrToInt64(RowData2);
      T2 := UnixToDateTime(UT);
      if not PosInSeconds then
        Result := FormatDateTime(Format, T1-T2)
      else
        Result := IntToStr(SecondsBetween(T2, T1));
    end
    else
      Result := '';
  except
    Result := 'Error';
  end;
end;

function TSageThread.GetEndTime(RecNo: Integer; Format: string): string;
var
  RowData1, RowData2: string;
  Row1, Row2: Integer;
  UT: Int64;
  T1, T2: TDateTime;
  ShowLength: Boolean;
  ShowLengthInSeconds: Boolean;
begin
  ShowLength := False;
  ShowLengthInSeconds := False;
  if Format = '' then
    Format := 'l#h:nn:ss';
  if Pos('#', Format) > 0 then
  begin
    ShowLength := AnsiLowerCase(Format[1]) = 'l';
    if Pos('#', Format) > 1 then
      ShowLengthInSeconds := AnsiLowerCase(Format[2]) = 's';
    Format := Copy(Format, Pos('#', Format)+1, MAXINT);
  end;
  if (Length(Format) > 0) and (ANSILowerCase(Format[1]) = 'l') then
  begin
    ShowLength := True;
    Format := Copy(Format, 2, MAXINT);
  end;
  Row1 := 6;
  Row2 := 5;
  if RecNo > 0 then
  begin
    Row1 := 15 + (RecNo-1) * 6;
    Row2 := 14 + (RecNo-1) * 6;
  end;
  try
    FCriticalSection.Enter;
    if Row1 < Data.Count then
    begin
      RowData1 := Data[Row1];
      RowData2 := Data[Row2];
    end
    else
    begin
      RowData1 := '';
      RowData2 := '';
    end;
  finally
    FCriticalSection.Leave;
  end;
  RowData1 := Trim(Copy(RowData1, Pos(':', RowData1)+1, MAXINT));
  if Length(RowData1) > 3 then
    RowData1 := Copy(RowData1, 1, Length(RowData1)-3); // Remove milliseconds
  RowData2 := Trim(Copy(RowData2, Pos(':', RowData2)+1, MAXINT));
  if Length(RowData2) > 3 then
    RowData2 := Copy(RowData2, 1, Length(RowData2)-3); // Remove milliseconds
  try
    if RowData1 <> '' then
    begin
      UT := StrToInt64(RowData1);
      T1 := UnixToDateTime(UT);
      UT := StrToInt64(RowData2);
      T2 := UnixToDateTime(UT);
      if ShowLength then
        if ShowLengthInSeconds then
          Result := IntToStr(SecondsBetween(T1, T2))
        else
          Result := FormatDateTime(Format, T1-T2)
      else
        Result := FormatDateTime(Format, IncMinute(T1, -GetLocalTZBias));
    end
    else
      Result := '';
  except
    Result := 'Error';
  end;
end;

function TSageThread.GetStartTime(RecNo: Integer;
  Format: string): string;
var
  RowData: string;
  Row: Integer;
  UT: Int64;
  T: TDateTime;
begin
  Row := 5;
  if RecNo > 0 then
    Row := 14 + (RecNo-1) * 6;
  try
    FCriticalSection.Enter;
    if Row < Data.Count then
      RowData := Data[Row]
    else
      RowData := '';
  finally
    FCriticalSection.Leave;
  end;
  RowData := Trim(Copy(RowData, Pos(':', RowData)+1, MAXINT));
  RowData := Copy(RowData, 1, Length(RowData)-3); // Remove milliseconds
  try
    if RowData <> '' then
    begin
      UT := StrToInt64(RowData);
      T := IncMinute(UnixToDateTime(UT), -GetLocalTZBias);
      Result := FormatDateTime(Format, T);
    end
    else
      Result := '';
  except
    Result := 'Error';
  end;
end;

initialization


finalization

end.


